"""
Simple test API for stationmeta management
"""

from typing import List
from fastapi import APIRouter, Depends, HTTPException, Body
from sqlalchemy.orm import Session, sessionmaker
from sqlalchemy.engine import Engine
from . import crud, schemas
from toardb.utils.database import get_db, get_engine

router = APIRouter()

# plain views to post and get stationmeta

#get all entries of table stationmeta
@router.get('/stationmeta_core/', response_model=List[schemas.StationmetaCore])
def get_all_stationmeta_core(skip: int = 0, limit: int = None, db: Session = Depends(get_db)):
    stationmeta_core = crud.get_all_stationmeta_core(db, skip=skip, limit=limit)
    return stationmeta_core

# the same as above, but nested view
#get all entries of table stationmeta
@router.get('/stationmeta/', response_model=List[schemas.Stationmeta])
def get_all_stationmeta(skip: int = 0, limit: int = None, db: Session = Depends(get_db)):
    stationmeta = crud.get_all_stationmeta(db, skip=skip, limit=limit)
    return stationmeta

#get all core metadata of one station
@router.get('/stationmeta_core/{station_code}', response_model=schemas.StationmetaCore)
def get_stationmeta_core(station_code: str, db: Session = Depends(get_db)):
    db_stationmeta_core = crud.get_stationmeta_core(db, station_code=station_code)
    if db_stationmeta_core is None:
        raise HTTPException(status_code=404, detail="Data not found.")
    return db_stationmeta_core

# the same as above, but nested view
#get all core metadata of one station
@router.get('/stationmeta/{station_code}', response_model=schemas.Stationmeta)
def get_stationmeta(station_code: str, db: Session = Depends(get_db)):
    db_stationmeta = crud.get_stationmeta(db, station_code=station_code)
    if db_stationmeta is None:
        raise HTTPException(status_code=404, detail="Data not found.")
    return db_stationmeta

@router.get('/stationmeta_changelog/{station_id}', response_model=List[schemas.StationmetaChangelog])
def get_stationmeta_changelog(station_id: int, db: Session = Depends(get_db)):
    db_changelog = crud.get_stationmeta_changelog(db, station_id=station_id)
    return db_changelog

#some more gets to be tested:
# - get stationmeta_global
# - get stationmeta_aux
# - ...

# the original post command!
#@router.post('/stationmeta/', response_model=schemas.StationmetaCreate)
# now the dry run!
@router.post('/stationmeta/')
# The following command was not working as long as the upload via Body was defined.
# See bug report: https://github.com/tiangolo/fastapi/issues/300
# (Although this seems to be fixed in the meantime, it is not working in my FastAPI version.)
#def create_stationmeta_core(stationmeta_core: schemas.StationmetaCoreCreate, db: Session = Depends(get_db)):
def create_stationmeta_core(stationmeta: schemas.StationmetaCreate = Body(..., embed = True), db: Session = Depends(get_db), engine: Engine = Depends(get_engine)):
    for station_code in stationmeta.codes:
        db_stationmeta_core= crud.get_stationmeta_core(db, station_code=station_code)
        if db_stationmeta_core:
            raise HTTPException(status_code=400, detail="Station already registered.")
# the original post command!
#   return crud.create_stationmeta(db=db, engine=engine, stationmeta=stationmeta)
# now the dry run!
    response=crud.create_stationmeta(db=db, engine=engine, stationmeta=stationmeta)
    msg = response.body.decode('utf-8')
    raise HTTPException(status_code=200, detail=str(msg))

@router.patch('/stationmeta/{station_code}', response_model=schemas.StationmetaPatch)
def patch_stationmeta_core(station_code: str, description: str, stationmeta: schemas.StationmetaPatch = Body(..., embed = True), db: Session = Depends(get_db)):
    db_stationmeta = crud.get_stationmeta(db, station_code=station_code)
    if db_stationmeta is None:
        raise HTTPException(status_code=404, detail="Station for patching not found.")
    return crud.patch_stationmeta(db=db, description=description, station_id=db_stationmeta.id, stationmeta=stationmeta)

@router.patch('/stationmeta/delete_field/{station_code}', response_model=schemas.StationmetaPatch)
def patch_stationmeta_core(station_code: str, field: str, db: Session = Depends(get_db)):
    db_stationmeta = crud.get_stationmeta(db, station_code=station_code)
    if db_stationmeta is None:
        raise HTTPException(status_code=404, detail="Station for deleting field not found.")
    return crud.delete_stationmeta_field(db=db, station_id=db_stationmeta.id, field=field)
